<?php

/**
 * Class ModActiveRecord is the base class for classes representing relational data.
 * It provides methods to check if attributes were modified
 */
class ModActiveRecord extends CActiveRecord
{
	protected $_modified = array();

	/**
	 * Sets the named attribute value.
	 * Also mark this attribute as modified
	 * @see CActiveRecord::setAttribute
	 */
	public function setAttribute($name,$value)
	{
		$oldValue = $this->getAttribute($name);
		if ($retValue = parent::setAttribute($name,$value))
		{
			if (!isset($this->_modified[$name]))
				$this->_modified[$name] = array('new' => $value, 'old' => $oldValue);
			else
				$this->_modified[$name]['new'] = $value;

			if ($this->_modified[$name]['new'] == $this->_modified[$name]['old'])
				unset($this->_modified[$name]);

			// clear relations that are depends on modified attribute
			/** @var \CActiveRelation $relation */
			foreach($this->getMetaData()->relations as $relationName => $relation)
				if ($name == $relation->foreignKey)
					unset($this->$relationName);
		}
		return $retValue;
	}

	/**
	 * Returns true if attribute $name was modified. If $name not set, returns true if any attribute were modified
	 */
	public function getIsModified($name = null)
	{
		if (is_null($name))
			return !empty($this->_modified);
		else
			return isset($this->_modified[$name]);
	}

	/**
	 * Resets modified marks of attribute named $name or all attributes
	 */
	public function resetModified($name = null)
	{
		if (is_null($name))
			$this->_modified = array();
		else
			unset($this->_modified[$name]);
	}

	/**
	 * Returns pair of original and modified values for attribute named $name or list of pairs for all modified attributes
	 *
	 * @return array
	 */
	public function getModified($name = null)
	{
		if (is_null($name))
			return $this->_modified;
		elseif (isset($this->_modified[$name]))
			return $this->_modified[$name];
		else
			return false;
	}

	/**
	 * Saves the current record. After that reset modified marks
	 *
	 * @see CActiveRecord::save
	 */
	public function save($runValidation=true, $attributes=null)
	{
		if ($retVal = parent::save($runValidation, $attributes))
		{
			$this->resetModified();
		}
		return $retVal;
	}
}